program pe;     { simple pascal editor }
{ written by Russell Whitaker
  v 0.1 - Jan 2000
  v 0.2 - Feb 2002 released under GPL
  v 0.3 - Oct 2002 search/replace: y/n/a/q
                   lineinsert: activated
  v 0.4 - Jan 2003 added "w" to search options
  v 0.5 - Feb 2003 changed test for "w"
  v 0.6 - Mar 2003 changed "w" to "i", fixed bug, fixed most warns.
                   }
uses CRT, GPC, GPCUtil,    {gpc supplied units}
     StringShift, pePrint,
     peBuffer, { 16 procs & functions, starting with BB_ }
     peMenu;   { menu items, dialogs, messages }

{$define Xmax GetXMax}
{$define Ymax GetYMax}

const
  MinScreenSizeX : integer = 60;
  MinScreenSizeY : integer = 22;

var
  i,j   : integer = 0;                  {temp vars}
  inp   : TKey;                         {input key}
  InsertMode : boolean = true;          {insert mode/ overwrite mode toggle}
  state2 : WinState;                    {windows}
  Xoffset : integer = 0;                {horiz displacement}
  Yoffset : integer = 0;                {index of first line in display}
  CurLine : lline;                      {Current Line = work space}
  UpdateFlag : boolean = false;         {true = new data in CurLine}
  MarkStart : integer = 0;              {start of marked block}
  MarkEnd : integer = 0;                {end of marked block}
  Pattern : NameString;                 {find string}
  RepPattern : NameString;              {replace string}
  CaseFlag,                             {true = case sensitive}
  ReplaceFlag,                          {true = replace}
  WordFlag : boolean;                   {true = find exact word}
  StartLine,
  EndLine : integer;                    {for search/continue}
  Found   : integer = 0;                { search/repl stats }
  Replace: integer = 0;

procedure DisplayLine( Index: integer );
var
 SetColor : TTextAttr;
 Dline : string[ Xmax ];
 i : integer;
begin
  if (( MarkEnd >= ( Index + Yoffset )) and
      ( MarkStart <= ( Index + Yoffset ))) then
    SetColor := Color[ 1 ]
  else
    SetColor := Color[ 3 ];
  Dline := copy( BB_ReadLine( Index + Yoffset ), Xoffset + 1, Xmax );
  WriteStrAt( 1, Index, Dline, SetColor );
  for i := 1 to Xmax do
  begin
    if i > Length( Dline ) then break;
    if  Dline[ i ] = chTab then
      ChangeTextAttr( i, Index, 1, Red);
  end;
end;

procedure ResetCurLine;
begin
  FillChar( CurLine[ 1 ], CurLine.Capacity, ' ');
  SetLength( CurLine, 0 );
  WriteCharAt( 1, WhereY, Xmax, ' ', TextAttr );
  UpdateFlag := false;
  DisplayLine( WhereY );
end;

function Trimit( Start: integer ): integer;  {trim blanks, return index}
var
  i : integer;
begin
  i := Start;
  while ( i > 1 ) and then
        ( CurLine[ i ] = ' ') or ( CurLine[ i ] = chr( kbTab )) do
    dec( i );
  Trimit := i;
end;

procedure WriteEditWindow;
var
  i, j,
  x, y : integer;
begin
  x := WhereX;
  y := WhereY;
  TextBackground( Blue );
  ClrScr;
  GotoXY( x, y );
  if ( Yoffset + Ymax ) < BB_Count then              {not EOF}
    j := Ymax
  else
    j := BB_Count - Yoffset;
  for i := 1 to j do
    DisplayLine( i );

  if UpdateFlag then
    for i := 1 to Xmax do
      if CurLine[ i + Xoffset ] = chTab then
        WriteCharAt( i, WhereY, 1, CurLine[ i + Xoffset ], Red )
      else
        WriteCharAt( i, WhereY, 1, CurLine[ i + Xoffset ], TextAttr );
end;

{------------------------------Cursor Moves------------------------------------}

procedure CursorDown;
var
  i : integer;
begin
  if ( Yoffset + WhereY ) <= BB_Count then        {allows cursor at 1st empty line}
    if WhereY < Ymax then
      GotoXY( WhereX, WhereY + 1 )
    else
    begin
      i := WhereX;
      writeln;
      inc( Yoffset );
      DisplayLine( WhereY );       {Ymax s/b = WhereY}
      GotoXY( i, WhereY );
    end;
end;

procedure CursorEnd;
var
  i : integer = CurLine.Capacity;
begin
  if not UpdateFlag then                         {get line to work on}
  begin
    ResetCurLine;
    CurLine := BB_ReadLine( WhereY + Yoffset );
  end;
  while ( i > 0 ) and ( CurLine[ i ] = ' ') do   {find end}
    dec( i );
  if ( i >= Xoffset ) and ( i < ( Xmax + Xoffset )) then  {end on screen}
    i := i - Xoffset
  else                                           {new Xoffset needed}
  begin
    if i < Xmax then                          {low boundry}
      Xoffset := 0
    else
      if i > ( CurLine.Capacity - Xmax ) then   {high boundry}
        Xoffset := CurLine.Capacity - Xmax
      else                                       {mid screen}
        Xoffset := i - ( Xmax div 2 );
    i := i - Xoffset;
    WriteEditWindow;
  end;
  GotoXY( i + 1, WhereY );
end;

procedure CursorHome;
begin
  if Xoffset > 0 then
  begin
    Xoffset := 0;
    WriteEditWindow;
  end;
  GotoXY( 1, WhereY );
end;

procedure CursorLeft;
begin
  if WhereX > 1 then
    GotoXY( WhereX - 1, WhereY )
  else
    if Xoffset > 0 then
    begin
      dec( Xoffset );
      WriteEditWindow;
      GotoXY( 1, WhereY );
    end;
end;

procedure CursorRight;
begin
  if WhereX < Xmax then
    GotoXY( WhereX + 1, WhereY )
  else
    if Xoffset < ( CurLine.Capacity - Xmax ) then
    begin
      inc( Xoffset );
      WriteEditWindow;
      GotoXY( Xmax, WhereY );
    end;
end;

procedure CursorUp;
begin
  ResetCurLine;
  if WhereY > 1 then              { not at top of screen}
    GotoXY( WhereX, WhereY - 1 )
  else
    if Yoffset > 0 then
    begin
      InsLine;
      dec( Yoffset );
      DisplayLine( 1 );
    end;
end;

{--------------------------Additional functions-------------------}

procedure PageDown;
var
  y : integer;
begin
  ResetCurLine;
  if( Yoffset + 20 ) < BB_Count then
     Yoffset := Yoffset + 20;
  if ( Yoffset + WhereY ) > BB_Count then
    y := BB_Count - Yoffset + 1                 {allows go past end}
  else
    y := WhereY;
  WriteEditWindow;
  GotoXY( WhereX, y );
end;

procedure PageUp;
begin
  ResetCurLine;
  if Yoffset > Ymax then
    Yoffset := Yoffset - 20
  else
    Yoffset := 0;
  WriteEditWindow;
end;

procedure MoveCursor2Target;
begin
  MoveCursorMsg;
  repeat
    inp := ReadKeyWord;
    case inp of
      kbUp   : CursorUp;
      kbDown : CursorDown;
      kbPgUp : PageUp;
      kbPgDn : PageDown;
      kbEnter : ;
    otherwise
      Beep;
    end;
    SaveWin( state2 );
    RestoreWin( MenuWin );
    WriteStrAt( 68, 1, '    ', TextAttr );       {blank out 4 spaces}
    WriteStrAt( 68, 1, Int2Str( WhereY + Yoffset ), TextAttr );
    RestoreWin( state2 );
  until inp = kbEnter;
end;

procedure BackSpace;
begin
  if not UpdateFlag then
  begin
    CurLine := BB_ReadLine( WhereY + Yoffset );
    UpdateFlag := true;
  end;
  CursorLeft;
  CurLine[ WhereX + Xoffset ] := ' ';
  WriteCharAt( WhereX, WhereY, 1, ' ', TextAttr );
end;

procedure Delete;
begin
  if not UpdateFlag then
  begin
    CurLine := BB_ReadLine( WhereY + Yoffset );
    UpdateFlag := true;
  end;
  if ( WhereX + Xoffset ) < Length( CurLine ) then
    MoveLeft ( CurLine[ WhereX + Xoffset + 1 ], CurLine[ WhereX + Xoffset ],
               Length( CurLine ) - WhereX - Xoffset + 1 )
  else
    CurLine[ WhereX + Xoffset ] := ' ';
  WriteEditWindow;
end;

procedure DeleteToEOL;
var
  i : integer;
begin
  if not UpdateFlag then
  begin
    CurLine := BB_ReadLine( WhereY + Yoffset );
    UpdateFlag := true;
  end;
  for i := ( WhereX + Xoffset ) to CurLine.Capacity do
    CurLine[ i ] := ' ';
  for i := 1 to Xmax do
    WriteCharAt( i, WhereY, 1, CurLine[ Xoffset + i ], TextAttr );
end;

procedure GoToLine( Linenum: integer );
var
  i: integer;
begin
  if Linenum > BB_Count then
    i := BB_Count + 1
  else
    i := Linenum;
  if i < (( Ymax div 2 ) + 1 ) then
    Yoffset := 0
  else
    Yoffset := i - (( Ymax div 2 ) + 1 );
  WriteEditWindow;
  GotoXY( 1, i - Yoffset );
end;

procedure InsertLine;
begin
  BB_InsertLine( WhereY + Yoffset, '' );    {inserts empty line in BigBuffer}
  CursorHome;           {sets x to col 1}
  WriteEditWindow;      {update screen}
end;

procedure InsertFile;
var
  s : string(80);
begin
  s := GetNameDialog('Insert File');
  if Length( s ) > 0 then
    BB_InsertFile( s, WhereY + Yoffset );
end;

procedure Stats;
var
  i : integer;
  numlines  : integer = 0;
  Length    : integer = 0;
  StartLine : integer = 1;
begin
  for i := 1 to BB_Count do
    if BB_Len( i ) > Length then
    begin
      Length := BB_Len( i );
      StartLine := i;
      numlines := 1;
    end
    else
      if BB_Len( i ) = Length then
      inc( numlines );
   StatsMsg( Length, StartLine, numlines );
   readln;
   WriteEditWindow;
end;

{--------------------------------Submenu items--------------------------------}
{--------------------------------File-----------------------------------------}
procedure FileCmds;
var
  ch : char;
begin
 ch := UpCase( FileSubMenu );
 case ch of
  'I' : InsertFile;
  'P' : PrintFile( filenam );
  'S' : Stats;
  otherwise
    Beep;
  end;
  WriteEditWindow;
end;


{--------------------------------Block----------------------------------------}
procedure BlockMark;
var
  inp : word;
  Anchor : integer;
begin
  if ( WhereY + Yoffset ) <= BB_Count then
  begin
    MarkStart := WhereY + Yoffset;
    MarkEnd := MarkStart;
    Anchor := MarkStart;
    repeat
      MarkBlockMsg;
      inp := ReadKeyWord;
      if inp = kbUp then
        CursorUp
      else
        if ( inp = kbDown ) and ((WhereY + Yoffset) < BB_Count ) then
          CursorDown;
      if (WhereY + Yoffset) <= Anchor then
        MarkStart := WhereY + Yoffset;
      if (WhereY + Yoffset) >= Anchor then
        MarkEnd := WhereY + Yoffset;
      WriteEditWindow;
    until inp = kbEnter;
  end
  else
    Beep;
end;

procedure BlockCopy;
var
  BlockLen: integer;
begin
  MoveCursor2Target;
  BlockLen := MarkEnd - MarkStart + 1;
  if ((( WhereY + Yoffset ) <= MarkEnd ) and
      (( WhereY + Yoffset ) >= MarkStart )) then
    Beep
  else
  begin
    if (( WhereY + Yoffset ) < MarkStart ) then
      BB_CopyBlock( MarkStart + BlockLen, WhereY + Yoffset, BlockLen )
    else
      BB_CopyBlock( MarkStart, WhereY + Yoffset, BlockLen );
    MarkStart := WhereY + Yoffset;
    MarkEnd := MarkStart + BlockLen -1;
    WriteEditWindow;
  end;
end;

procedure BlockUnmark;
begin
  MarkStart := 0;
  MarkEnd := 0;
  WriteEditWindow;
end;

procedure BlockDelete;
begin
  BB_DeleteBlock( MarkStart, MarkEnd );
  if (WhereY + Yoffset) > MarkStart then
      if (WhereY + Yoffset) > MarkEnd then
        GoToLine( WhereY + Yoffset + MarkStart - MarkEnd -1 )
      else
        GoToLine( MarkStart );
  BlockUnmark;
end;

procedure BlockMove;
var
  SaveY,
  BlockLen : integer;
begin
  MoveCursor2Target;
  BlockLen := MarkEnd - MarkStart + 1;
  BB_MoveBlock( MarkStart, WhereY + Yoffset, BlockLen );
  if ( WhereY + Yoffset ) > MarkStart then
    MarkStart := WhereY + Yoffset - BlockLen
  else
    MarkStart := WhereY + Yoffset;
  MarkEnd := MarkStart + BlockLen - 1;
  SaveY := WhereY;
  ClrScr;
  GotoXY( 1,SaveY );
  WriteEditWindow;
end;

procedure BlockShift;
begin
  ShiftMsg;
  repeat
    inp := ReadKeyWord;
    case inp of
      kbLeft : BB_ShiftBlock( MarkStart, MarkEnd, 1, false );
      kbRight: BB_ShiftBlock( MarkStart, MarkEnd, 1, true );
      kbShTab: BB_ShiftBlock( MarkStart, MarkEnd, 4, false );
      kbTab  : BB_ShiftBlock( MarkStart, MarkEnd, 4, true );
      kbEnter: ;
    otherwise
      Beep;
    end;
    WriteEditWindow;
  until inp = kbEnter;
end;

procedure BlockWrite;
var
  OutFileName: NameString;
  OutFile    : text;
  i : integer;
begin
  OutFileName := GetNameDialog('Create/Append to File');
  if Length( OutFileName ) > 0 then
  begin
    Assign( OutFile, OutFileName );
    Append( OutFile );
    for i := MarkStart to MarkEnd do
      writeln( OutFile, BB_ReadLine( i ));
    Close( OutFile );
  end;
end;

procedure BlockCmds;
var
  ch : char;
begin
  if MarkStart = 0 then
    BlockMark;
  ch := UpCase( BlockSubMenu );
  case ch of
  'B' : begin
          BlockUnmark;
          BlockMark;    {ok}
          end;
  'C' : BlockCopy;    {move cursor first}
  'D' : BlockDelete;
  'M' : BlockMove;    {= copy, delete}
  'S' : BlockShift;
  'U' : BlockUnmark;  {ok}
  'W' : BlockWrite;
  otherwise
    Beep;
  end;
  WriteEditWindow;
end;


procedure BlockFix( Index: integer; AddedLine: boolean );
begin
  if AddedLine then
  begin
    if Index < MarkStart then inc( MarkStart );
    if Index < MarkEnd then inc( MarkEnd );
  end
  else
  begin
    if Index < MarkStart then dec( MarkStart );
    if Index < MarkEnd then dec( MarkEnd );
  end;
end;
 

{-----------------------------------Line--------------------------------------}
procedure LineAppend;
var
  tmp : lline;
  indx: integer;
  StartPos : integer = 1;
begin
  indx := WhereY + Yoffset;
  if( indx > 1 ) and ( indx <= BB_Count ) then
  begin
    tmp := BB_ReadLine( indx );
    BB_DeleteLine( indx );
    if MarkStart > 0 then
      BlockFix( indx, false );
    CursorUp;
    CurLine := BB_ReadLine( WhereY + Yoffset );
    CursorEnd;
    while ( StartPos < Length( tmp )) and ( tmp[ StartPos ] = ' ' ) do
      inc( StartPos );
    CurLine := concat( CurLine, copy( tmp, StartPos, Length( tmp ) - StartPos + 1));
    BB_WriteLine( WhereY + Yoffset, CurLine );
    WriteEditWindow;
  end
  else
    Beep;
end;

procedure LineCopy;
var
  FromIndex, ToIndex : integer;
begin
  FromIndex := WhereY + Yoffset;
  if FromIndex > BB_Count then
    Beep
  else
  begin
    MoveCursor2Target;
    ToIndex := WhereY + Yoffset;
    BB_InsertLine( ToIndex, BB_ReadLine( FromIndex ));
    if MarkStart > 0 then
      BlockFix( ToIndex, true );
    WriteEditWindow;
  end;
end;

procedure LineDup;
var
  i: integer;
begin
  i := WhereY + Yoffset;
  CursorDown;
  BB_InsertLine( i+1, BB_ReadLine( i ));
  if MarkStart > 0 then
    BlockFix( i, true );
  WriteEditWindow;
end;

procedure LineErase;
begin
  if ( WhereY + Yoffset ) <= BB_Count then
  begin
    ResetCurLine;
    BB_DeleteLine( WhereY + Yoffset );
    if MarkStart > 0 then
      BlockFix( WhereY + Yoffset, false );
    WriteEditWindow;
  end;
  if ( WhereY + Yoffset ) > BB_Count then     {deleted last line}
    CursorUp;
end;

procedure LineMove;
var
  FromIndex, ToIndex : integer;
begin
  FromIndex := WhereY + Yoffset;
  if FromIndex > BB_Count then
    Beep
  else
  begin
    MoveCursor2Target;
    ToIndex := WhereY + Yoffset;
    if ToIndex > BB_Count then
      ToIndex := BB_Count;
    BB_MoveLine( FromIndex, ToIndex );
    if MarkStart > 0 then
    begin
      BlockFix( ToIndex, true );
      BlockFix( FromIndex, false );
    end;
    WriteEditWindow;
  end;
end;

procedure LineSplit;
var
  StartPos : integer = 1;
  CursorPos,
  LastPos : integer;
begin
  if not UpdateFlag then
    CurLine := BB_ReadLine( WhereY + Yoffset )
  else
    UpdateFlag := false;                       {"Enter" not needed}
  while ( StartPos < Length( CurLine )) and ( CurLine[ StartPos ] = ' ' ) do
    inc( StartPos );
  CursorPos := WhereX + Xoffset;
  LastPos := Trimit( CursorPos - 1 );       {defined first segment}
  {$X+}
  SetLength( CurLine, LastPos );
  {$X-}
  BB_WriteLine( WhereY + Yoffset, CurLine );

  CursorDown;
  BB_InsertLine( WhereY + Yoffset, '' );
  LastPos := Trimit( CurLine.Capacity );      {end of 2nd segment}
  MoveLeft( CurLine[ CursorPos ], CurLine[ StartPos ], LastPos - CursorPos + 1);
  {$X+}
  SetLength( CurLine, LastPos - CursorPos + StartPos );
  {$X-}

  BB_WriteLine( WhereY + Yoffset, CurLine );
  ResetCurLine;
  if MarkStart > 0 then
    BlockFix( StartPos, true );
  WriteEditWindow;
  GotoXY( StartPos, WhereY );
end;

(*---------------------------Search-------------------------------------------
  find outer loop:
    uses start - end from submenu
  find inner loop:
    uses
       c:  case sens/insens;  r: replace;  (y/q dialog) or (y/n/a/q) dialog
-----------------------------------------------------------------------------*)

procedure FindCtrl( FromLine, ToLine: integer );
var
  ch: char;
  i,j : integer;
  AllFlag: boolean = false;
  done: boolean;

  function doline( Index: integer ): boolean;
 
    function ckchar( c: char ): boolean;
    begin
      if ( IsAlphaNum( c ) or ( c = '_' ) ) then
        ckchar := false
      else
        ckchar := true;
     end;

  var
    i: integer;
    ok,
    quit: boolean;
  begin
    CurLine := BB_ReadLine( Index );
    quit := false;
    i := 1;
    repeat
      {search for pattern match in CurLine}
      repeat
        if CaseFlag then
          i := PosFrom( Pattern, CurLine, i )
        else
          i := PosFrom( Pattern, LoCaseStr( CurLine ), i );
        if WordFlag and ( i > 0 ) then
        {test: if string starts line or char preceding isn't [A..Z,a..z,0..9,_]
         and either string ends line or char after isn't [A..Z,a..z,0..9,_].  }
        begin
          if( ( i = 1 ) or ckchar( char( CurLine[i-1]))
            ) and (
              (( i + Length( Pattern )) > Length( CurLine ))
              or ckchar( char( CurLine[ i + Length( Pattern )]))
            ) then
            ok := true
          else
          begin
            inc( i );
            ok := false;
          end;
        end
        else
          ok := true;
      until ok;
      if i > 0 then           {found match}
      begin
        inc( Found );
        if ( i + Length( Pattern )) > Xmax then
          Xoffset := i + Length( Pattern ) - Xmax
        else
          Xoffset := 0;
        GoToLine( Index );
        ChangeTextAttr( i - Xoffset, WhereY, Length( Pattern ), Color[ 2 ]);
        SaveWin( state2 );
        RestoreWin( MenuWin );
        WriteStrAt( 68, 1, '    ', TextAttr );       {blank out 4 spaces}
        WriteStrAt( 68, 1, Int2Str( Index ), TextAttr ); {line number}
        WriteStrAt( 78, 1, '   ', TextAttr );        {blank out 3 spaces}
        WriteStrAt( 78, 1, Int2Str( i ), TextAttr ); {col number}
        RestoreWin( state2 );
        if ReplaceFlag then
        begin
          if not AllFlag then
          begin
            ch := YesNoAllQuitDialog( 'Replace?');
            if ch = 'a' then
            begin
              AllFlag := true;
              ch := 'y';
            end
            else
              if ch = 'q' then
              begin
                i := 0;
                quit := true;
              end;
          end;
          if ch = 'y' then
          begin
            inc( Replace );
            if Length( Pattern ) > Length( RepPattern ) then
              StringShiftLeft( CurLine, i, Length( Pattern ) - Length( RepPattern ))
            else
              if Length( Pattern ) < Length( RepPattern ) then
                StringShiftRight( CurLine, i, Length( RepPattern ) - Length( Pattern ));
            move( RepPattern[ 1 ], CurLine[ i ], Length( RepPattern ));
            BB_WriteLine( Index, CurLine );
          end;
          inc( i );
        end
        else
        begin
          ch := YesQuitDialog('Again?');
          if ch = 'y' then
            inc( i )
          else
          begin
            i := 0;
            quit := true;
          end;
        end;
      end;
    until (i = 0) or quit;
    doline := quit;
  end;

begin
  done := false;
  if FromLine < ToLine then
  begin
    for i := FromLine to ToLine do
    begin
      if BB_Len( i ) >= Length( Pattern ) then
        done := doline( i );
      if done then
      begin
        if i = ToLine then
          StartLine := 0
        else
        begin
          StartLine := i;         {saves for search/continue}
          EndLine := ToLine;
          break;
        end;
      end;
    end;
  end
  else
  begin
    for i := FromLine downto ToLine do
    begin
      if BB_Len( i ) >= Length( Pattern ) then
        done := doline( i );
      if done then
      begin
        if i = ToLine then
          StartLine := 0
        else
        begin
          StartLine := i;
          dec( StartLine );
          EndLine := ToLine;
          break;
        end;
      end;
    end;
  end;
  WriteEditWindow;
  for j:= BB_Len( i ) to  CurLine.Capacity do     {clean out garbage}
    CurLine[ j ] := ' ';
end;

{----------------------------main menu items----------------------------------}
procedure LineCmds;
var
  ch : char;
begin
  if ( WhereY + Yoffset ) <= BB_Count then
  begin
      ChangeTextAttr( 1, WhereY, BB_Len( WhereY + Yoffset ) - Xoffset, Color[ 4 ] );
    ch := UpCase( LineSubMenu );
    case ch of
      'A' : LineAppend;
      'C' : LineCopy;
      'D' : LineDup;
      'E' : LineErase;
      'I' : InsertLine;
      'M' : LineMove;
      'S' : LineSplit;
    end;
  end
  else
    Beep;
  WriteEditWindow;
end;

procedure SearchCmds;
var
  ch : char;
  tempstr : string ( 7 );
begin
  ch := UpCase( SearchSubMenu );
  WriteEditWindow;
  if ( ch = 'C') then
  begin
    if ( StartLine > 0 ) then
      FindCtrl( StartLine, EndLine )
    else
      Beep;
  end
  else
    if ( ch = 'R' ) then
    begin
      ResultsMsg( Found, Replace );
      readln;
    end
    else
    begin
      if (ch = 'F') or (ch = 'U') or (ch = 'D') or (ch = 'B') then
      begin
        Found := 0;
        Replace := 0;
        Pattern :=  GetNameDialog('Search for');
        CaseFlag := false;
        ReplaceFlag := false;
        WordFlag := false;
        tempstr := OptsDialog;
        i := Length( tempstr );
        while( i > 0 ) do
        begin
          if UpCase( tempstr[ i ]) = 'C' then
            CaseFlag := true
          else
            if UpCase( tempstr[ i ]) = 'R' then
              ReplaceFlag := true
          else
            if UpCase( tempstr[ i ]) = 'I' then
              WordFlag := true;
          dec( i );
        end;
        if ReplaceFlag then
          RepPattern := GetNameDialog('Replace with');
        if not CaseFlag then
          Pattern := LoCaseStr( Pattern );
        case ch of
          'F' : FindCtrl( 1, BB_Count );                 {full file search}
          'U' : begin
                  if ( WhereY + Yoffset ) > 1 then
                    FindCtrl( WhereY + Yoffset - 1, 1 )  {cur line-1,  up }
                  else
                    Beep;
                end;
          'D' : FindCtrl( WhereY + Yoffset, BB_Count );  {cur line,  down}
          'B' : begin
                  if ( MarkStart = 0 ) then BlockMark;
                  FindCtrl( MarkStart, MarkEnd );          {block}
                end;
        end;
    end
    else
      Beep;
  end;
  WriteEditWindow;
end;

{-------------------------control-------------------------------}

procedure Enter;
var
  StartPos : integer;
begin
  ResetCurLine;
  Xoffset := 0;
  if InsertMode then
  begin
    StartPos := BB_Start( WhereY + Yoffset );
    CursorDown;
    InsertLine;
    if MarkStart > 0 then
      BlockFix( WhereY + Yoffset, true );
  end
  else
  begin
    CursorDown;
    StartPos := BB_Start( WhereY + Yoffset );
  end;
  if StartPos > Xmax then
  begin
    Xoffset := StartPos - 1;
    StartPos := 1;
  end;
  WriteEditWindow;
  GotoXY( StartPos, WhereY );
end;


{-------------------------Character handler----------------------------}
procedure HandleCH;
var
 SetColor : TTextAttr;
begin
  if not UpdateFlag then
  begin
    CurLine := BB_ReadLine( WhereY + Yoffset );
    UpdateFlag := true;
  end;
  if (( MarkEnd >= ( WhereY + Yoffset )) and
      ( MarkStart <= ( WhereY + Yoffset ))) then
    SetColor := Color[ 1 ]
  else
    SetColor := Color[ 3 ];

  { 1. cur pos > Length( CurLine ):     setlength, add char to end, cursor
right
    2. cur pos = Length( CurLine ):     add char to end, cursor right
    3. cur pos < Length( CurLine )
        insert mode:                    add char to end, right shift data,
                                        write char, cursor right
        overwrite mode:                 write char, cursor right }

  if ( WhereX + Xoffset ) > Length( CurLine ) then
  begin
    SetLength( CurLine, ( WhereX + Xoffset ));
  end
  else
  begin
    if InsertMode then
    begin
      CurLine := concat( CurLine, ' ' );
      MoveRight( CurLine[ WhereX + Xoffset ], CurLine[ WhereX + Xoffset + 1 ],
                 Length( CurLine ) - WhereX - Xoffset  );
    end;
  end;
  CurLine[ WhereX + Xoffset ] := Key2Char( inp );

  for i := 1 to Xmax do
    if  CurLine[ Xoffset + i ] = chTab then
      WriteCharAt( i, WhereY, 1, CurLine[ Xoffset + i ], Red )
    else
      WriteCharAt( i, WhereY, 1, CurLine[ Xoffset + i ], SetColor );
  CursorRight;

end;

{----------------------------Main -------------------------------}

begin
  if (ScreenSize.x < MinScreenSizeX ) or (ScreenSize.y < MinScreenSizeY) then
    writeln('Screen Size is too small (',MinScreenSizeX,'x',MinScreenSizeY,' min)')
  else
  begin
    InitMenuBar;
    InitEditWin;
    ResetCurLine;
    i := BB_getfile;
    if i > 0 then
      GoToLine( i )
    else
    begin
      WriteEditWindow;
      GotoXY( WhereX, 1 );          {start off at 1st line}
    end;
    inp := 0;
    while( inp <> kbAltX ) do
    begin
      i := WhereX + Xoffset;
      j := WhereY + Yoffset;
      SaveWin( state2 );
      RestoreWin( MenuWin );
      if InsertMode then
        WriteStrAt( 56, 1, 'Ins', TextAttr )
      else
        WriteStrAt( 56, 1, 'Ovl', TextAttr );
      WriteStrAt( 66, 1, '     ', TextAttr );      {blank out 5 spaces}
      WriteStrAt( 66, 1, Int2Str( j ), TextAttr ); {line number}
      WriteStrAt( 77, 1, '    ', TextAttr );       {blank out 4 spaces}
      WriteStrAt( 77, 1, Int2Str( i ), TextAttr ); {col number}
      RestoreWin( state2 );
      inp := ReadKeyWord;
      case inp of
      {cursor moves}
        kbHome   : CursorHome;
        kbLeft   : CursorLeft;
        kbRight  : CursorRight;
        kbBkSp   : BackSpace;
        kbDel    : Delete;
        kbEnd    : CursorEnd;
        kbShTab : Beep;
      {commands}
        kbAltDel  : DeleteToEOL;
        kbAltH    : begin
                      HelpMsg;
                      WriteEditWindow;
                    end;
        kbAltR    : ResetCurLine;
        kbIns     : InsertMode := not( InsertMode );        {toggle}
        kbShIns   : InsertLine;
      {data}
        $20 .. $7F, kbTab : HandleCH;
      otherwise
        begin
          if UpdateFlag then
          begin
            i := Trimit( CurLine.Capacity );
            SetLength( CurLine, i );
            BB_WriteLine( WhereY + Yoffset, CurLine );
            UpdateFlag := false;
          end;
          ResetCurLine;
          case inp of
            kbDown    : CursorDown;
            kbUp      : CursorUp;
            kbPgDn    : PageDown;
            kbPgUp    : PageUp;
            kbEnter   : Enter;
            kbAltB    : BlockCmds;
            kbAltF    : FileCmds;
            kbAltG    : GoToLine( GoToDialog );
            kbAltL    : LineCmds;
            kbAltS    : SearchCmds;
            kbAltX    : break;
          otherwise
            Beep;
          end;
        end;
      end;
    end;
    if BB_FileUpdate then
      WriteFileWin( filenam )
    else
      NoFileUpdateMsg;
  end;
end.
