Unit peMenu; {screen routines for pascal editor}
{written by Russell Whitaker <russ@ashlandhome.net>, Feb 2000
 updated July 2001
 10/17/2002: changed YesNoDialog to YesQuitDialog and YesNoAllQuitDialog
 11/11/2002: added continue to SearchSubMenu
  1/20/2003: added "w" to OptsDialog
  3/4/2003: changed "w" to "i" }
interface
uses GPC, CRT;

type
  NameString = string ( 80 );

  procedure InitMenuBar;
  procedure InitEditWin;

  function FileSubMenu: char;
  function BlockSubMenu: char;
  function LineSubMenu: char;
  function SearchSubMenu: char;
  function GetNameDialog( const Title: string ): NameString;
  function GoToDialog: integer;
  function OptsDialog: NameString;
  function YesQuitDialog( const Title: string ): char;
  function YesNoAllQuitDialog( const Title: string ): char;

  procedure MarkBlockMsg;
  procedure ShiftMsg;
  procedure MoveCursorMsg;
  procedure NoFileUpdateMsg;
  procedure ResultsMsg(Found, Replaced: integer );
  procedure StatsMsg(Length, Start, count: integer );
  procedure WriteFileWin( aFileName: string );
  procedure HelpMsg;

var
  MenuWin, EditWin, SubMenu : WinState;
  CurSize : TPoint;

type
  TColorPallet = array[ 1 .. 5 ] of TTextAttr;

const
  Color : TColorPallet = (( 16 * LightGray) + Black ,( 16 * LightGray ) + Red,
                          ( 16 * Blue ) + LightGray, ( 16 * Blue ) + Yellow,
                          ( 16 * LightGray) + Magenta );
implementation

uses GPCUtil, FrameWin, peBuffer;

const
  HelpText : array[1..18] of string( 48 ) = (
    'Command Line Options:',
    '  pe <File Name> <go to line number>',
    'Activate Main Menu items with ALT-letter',
    '  File submenu',
    '  Block submenu',
    '    If block is not marked then ALT-b starts',
    '    mark routine followed by block submenu',
    '  Line submenu',
    '  Search submenu',
    '  Help: one page',
    '  eXit: also asks exit options',
    '  Insert key toggles INSert/OVerlay modes',
    'Cursor moves:',
    '  Cursor up dn left right; page up dn; Home, End',
    'Hot keys Not on menu:',
    '  ALT-delete - deletes from cursor to EOL',
    '  ALT-R - restores current line from memory',
    '  Sh-Insert - insert new line at cursor');

  HelpTextColor : array[1..18] of integer =
    ( 4,3,4,3,3,3,3,3,3,3,3,3,4,3,4,3,3,3 );
 
{menu bar at top of screen}
procedure InitMenuBar;
begin
  MakeWin( MenuWin, 1, 1, CurSize.X, 1 );
  FillWin( '.', Color[ 1 ] );
  WriteStrAt( 1, 1, ' Pascal Editor', Color[ 2 ] );
  TextBackground( LightGray );
  TextColor( Black);
  WriteStrAt( 17, 1, 'F', Color[ 5 ] );
  WriteStrAt( 18, 1, 'ile  ', Color[ 1 ] );
  WriteStrAt( 23, 1, 'B', Color[ 5 ] );
  WriteStrAt( 24, 1, 'lock  ', Color[ 1 ] );
  WriteStrAt( 30, 1, 'L', Color[ 5 ] );
  WriteStrAt( 31, 1, 'ine   ', Color[ 1 ] );
  WriteStrAt( 36, 1, 'S', Color[ 5 ] );
  WriteStrAt( 37, 1, 'earch  ', Color[ 1 ] );
  WriteStrAt( 44, 1, 'H', Color[ 5 ] );
  WriteStrAt( 45, 1, 'elp  e', Color[ 1 ] );
  WriteStrAt( 51, 1, 'X', Color[ 5 ] );
  WriteStrAt( 52, 1, 'it   ', Color[ 1 ] );
  WriteStrAt( 60, 1, ' line ', Color[ 1 ] );
  WriteStrAt( 73, 1, 'col ', Color[ 1 ] );
  SaveWin( MenuWin );
end;

procedure InitEditWin;
begin
  TextBackground( Blue );
  TextColor( LightGray );
  MakeWin( EditWin, 1, 2, CurSize.X, CurSize.Y );
  FillWin( ' ', TextAttr );
end;

function FileSubMenu: char;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  HideCursor;
  MakeWin( SubMenu, 62, 3, 76, 8 );
  FrameWin('File Menu', SingleFrame );
  FillWin( ' ', TextAttr );
  WriteString( 'Insert',  1, 2 );
  WriteString( 'Print ',  2, 2 );
  WriteString( 'Stats ',  3, 2 );
  FileSubMenu := char( ReadKeyWord ); {avoids cursor readkey problem}
  RestoreWin( EditWin );
end;

function BlockSubMenu: char;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  HideCursor;
  MakeWin( SubMenu, 62, 3, 76, 11 );
  FrameWin('Block Menu', SingleFrame );
  FillWin( ' ', TextAttr );
  WriteString( 'mark Block',  1, 2 );
  WriteString( 'Unmark ',     2, 2 );
  WriteString( 'Copy',        3, 2 );
  WriteString( 'Delete',      4, 2 );
  WriteString( 'Move',        5, 2 );
  WriteString( 'Shift',       6, 2 );
  WriteString( 'Write file',  7, 2 );
  BlockSubMenu := char( ReadKeyWord ); {avoids cursor readkey problem}
  RestoreWin( EditWin );
end;

function LineSubMenu: char;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  HideCursor;
  MakeWin( SubMenu, 62, 3, 76, 11 );
  FrameWin('Line Menu', SingleFrame );
  FillWin( ' ', TextAttr );
  WriteString( 'Append',    1, 2 );
  WriteString( 'Copy',      2, 2 );
  WriteString( 'Duplicate', 3, 2 );
  WriteString( 'Erase',     4, 2 );
  WriteString( 'Insert',    5, 2 );
  WriteString( 'Move',      6, 2 );
  WriteString( 'Split',     7, 2 );
  LineSubMenu := char( ReadKeyWord );
  RestoreWin( EditWin );
end;

function SearchSubMenu: char;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  HideCursor;
  MakeWin( SubMenu, 62, 3, 77, 11 );
  FrameWin('Search Menu', SingleFrame );
  FillWin( ' ', TextAttr );
  WriteString( 'Full',       1, 2 );
  WriteString( 'Up',         2, 2 );
  WriteString( 'Down',       3, 2 );
  WriteString( 'Block',      4, 2 );
  WriteCharAt( 1,5,14, '-', Color[ 1 ] );
  WriteString( 'Continue',   6, 2 );
  WriteString( 'Results',    7, 2 );
  SearchSubMenu := char( ReadKeyWord );
  RestoreWin( EditWin );
end;

function GetNameDialog( const Title: string ): NameString;
var
  s : NameString;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 32, 3, 74, 5 );
  FrameWin( Title, SingleFrame );
  FillWin( ' ', Color[ 3 ] );
  readln( s );
  GetNameDialog := s;
  RestoreWin( EditWin );
end;

function GetNumber : integer;      {internal function}
var
  ch: char;
  i : integer = 0;
begin
  repeat
    ch := ReadKey;
    if ( ch >= '0' ) and ( ch <= '9' ) then
    begin
      i := ( 10 * i ) + integer( ch ) - 48;
      Write( ch );
    end;
  until ch = chEnter;
  GetNumber := i;
end;

function GoToDialog : integer;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 62, 3, 74, 5 );
  FrameWin('Go To Line', SingleFrame );
  FillWin( ' ', Color[ 3 ] );
  GoToDialog := GetNumber;
  RestoreWin( EditWin );
end;

function OptsDialog: NameString;
var
  s : NameString;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 62, 9, 75, 14 );
  FrameWin('Options', SingleFrame );
  FillWin( ' ', Color[ 3 ] );
  WriteStrAt( 1, 1, ' C', Color[ 5 ] );
  WriteStrAt( 3, 1, 'heck Case ', Color[ 1 ] );
  WriteStrAt( 1, 2, ' I', Color[ 5 ] );
  WriteStrAt( 3, 2, 'dentifier ', Color[ 1 ] );
  WriteStrAt( 1, 3, ' R', Color[ 5 ] );
  WriteStrAt( 3, 3, 'eplace    ', Color[ 1 ] );
  GotoXY( 1,4 );
  readln( s );
  RestoreWin( EditWin );
  OptsDialog := s;
end;

function YesQuitDialog( const Title: string ): char;
var
  ch : char;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 64, GetYMax - 4, 76, GetYMax - 2 );
  FrameWin( Title, SingleFrame );
  FillWin( ' ', Color[ 3 ] );
  write( 'Y/Q ' );
  repeat
    ch := LoCase( ReadKey );
  until (ch = 'y') or (ch = 'q') or (ch = 'n');
  RestoreWin( EditWin );
  YesQuitDialog := ch;
end;

function YesNoAllQuitDialog( const Title: string ): char;
var
  ch : char;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 60, GetYMax - 4, 76, GetYMax - 2 );
  FrameWin( Title, SingleFrame );
  FillWin( ' ', Color[ 3 ] );
  write( 'Y/N/A/Q ' );
  repeat
    ch := LoCase( ReadKey );
  until (ch = 'y') or (ch = 'n') or (ch = 'a' ) or (ch = 'q');
  RestoreWin( EditWin );
  YesNoAllQuitDialog := ch;
end;

procedure MarkBlockMsg;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 42, 3, 66, 8 );
  FrameWin('Marking Block', DoubleFrame );
  FillWin( ' ', Color[ 3 ] );
  WriteStrAt( 2,2, 'Use Up and Down Cursor keys', Color[ 4 ]);
  WriteStrAt( 2,3, '    Enter when done', Color[ 4 ]);
  RestoreWin( EditWin );
end;

procedure ShiftMsg;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 42, 3, 74, 8 );
  FrameWin('Shifting Block', DoubleFrame );
  FillWin( ' ', Color[ 3 ] );
  WriteStrAt( 2,1, 'Cursor Right/Left keys: 1 col' , Color[ 4 ]);
  WriteStrAt( 2,2, 'Tab/ShiftTab: move 4 columns' , Color[ 4 ]);
  WriteStrAt( 2,4, '    Enter when done', Color[ 4 ]);
  RestoreWin( EditWin );
end;

procedure MoveCursorMsg;
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 42, 3, 66, 8 );
  FrameWin('Information', DoubleFrame );
  FillWin( ' ', Color[ 3 ] );
  WriteStrAt( 2,2, 'Move Cursor to target', Color[ 4 ]);
  WriteStrAt( 2,3, 'line and press Enter', Color[ 4 ]);
  RestoreWin( EditWin );
end;

procedure NoFileUpdateMsg;
begin
  MakeWin( SubMenu, 42, 3, 66, 8 );
  FrameWin('Information', DoubleFrame );
  FillWin( ' ', Color[ 3 ] );
  WriteStrAt( 2,2, ' File not changed --', Color[ 4 ]);
  WriteStrAt( 2,3, ' so no update needed', Color[ 4 ]);
end;

procedure ResultsMsg(Found, Replaced: integer );
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 51, 3, 76, 9 );
  FrameWin('Search Statistics', DoubleFrame );
  FillWin( ' ', Color[ 3 ] );
  WriteStrAt( 3,2, 'Found:     ' + Int2Str( Found ), Color[ 4 ]);
  WriteStrAt( 3,3, 'Replaced:  ' + Int2Str( Replaced ), Color[ 4 ]);
  WriteStrAt( 4,5, 'Hit Enter when done', Color[ 4 ]);
  RestoreWin( EditWin );
end;

procedure StatsMsg(Length, start, count: integer );
begin
  SaveWin( EditWin );
  TextBackground( LightGray );
  TextColor( Black );
  MakeWin( SubMenu, 51, 3, 76, 10 );
  FrameWin('File Statistics', DoubleFrame );
  FillWin( ' ', Color[ 3 ] );
  WriteStrAt( 3,2, 'Longest line:   ' + Int2Str( Length ), Color[ 4 ]);
  WriteStrAt( 3,3, 'First occurs:   ' + Int2Str( Start ), Color[ 4 ]);
  WriteStrAt( 3,4, '      Occurs:   ' + Int2Str( count ), Color[ 4 ]);
  WriteStrAt( 4,6, 'Hit Enter when done', Color[ 4 ]);
  RestoreWin( EditWin );
end;

procedure WriteFileWin( aFileName: string );
var
  inp : integer;
  tempStr : string( 80 );
  F : text;
begin
  MakeWin( SubMenu, 2, 3, 50, 12);
  FrameWin('Write File Dialog', SingleFrame );
  FillWin( ' ', Color[ 3 ] );
  WriteStrAt( 2,2, 'Save changes to ' + aFileName + '?', Color[ 4 ]);
  WriteStrAt( 2,3, 'N = no, C = change name, Any other key = ok', Color[ 3 ]);
  GotoXY( 2, 4 );
  inp := ReadKeyWord;
  if not ( UpCase( char( inp )) = 'N' ) then
  begin
    if UpCase( char( inp )) = 'C' then
    begin
      WriteStrAt( 2,4, 'Input new file name:', Color[ 4 ]);
      GotoXY( 2, 5);
      readln( tempStr );
    end
    else
      tempStr := aFileName;
    writeln;
    if FileExists( tempStr ) then
    begin
      if FileExists( tempStr + '~' ) then
      begin
        Assign( F, tempStr + '~' );
        Erase( F );
        writeln( '   Deleted old backup ');
      end;
      Assign( F, tempStr );
      Rename( F, tempStr + '~' );
      writeln( '   Made new backup ');
    end;
    BB_Putfile( tempStr );
    writeln( '   Wrote file' );
  end
  else
  begin
    writeln;
    writeln( '   Changes not saved ');
  end;
end;

procedure HelpMsg;
var
  i : integer;
begin
  i := (CurSize.X - 52) div 2;
  SaveWin( EditWin );
  MakeWin( SubMenu, i, 2, i + 52, 22 );
  FrameWin('HELP: hit Enter to exit', DoubleFrame );
  FillWin( ' ', Color[ 3 ] );
  HideCursor;
  for i := 1 to 19 do
    WriteStrAt( 2,i, HelpText[ i ], Color[ HelpTextColor[ i ]] );
  readln;
  RestoreWin( EditWin );
end;

to begin do
    CurSize := ScreenSize;

end.
